﻿package com.nosuch {
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.Sprite;
	import flash.geom.Rectangle;
	import flash.events.*;
	import flash.utils.*;
	import flash.ui.Keyboard;
	
	public class CanvasController extends Sprite {
		
		private var _classList:Array;
		private var _itemList:Array;
		
		public var baseRotation:Number;
		public var randomRotation:Number;
		public var minAlpha:Number;
		public var maxAlpha:Number;
		public var minScale:Number;
		public var maxScale:Number;
		public var canvasBounds:Rectangle;
		public var killItem:Boolean;
		
		public function CanvasController(classNameList:Array = null):void {
			_itemList = new Array();
			if (classNameList != null) {
				setClasses(classNameList);
			}
			killItem = false;
			addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
		}
		
		public function setClasses(classNameList:Array):void {
			var i:uint;
			var max:uint = classNameList.length;
			
			_classList = new Array();
			for (i=0; i<max; i++) {
				_classList[i] = getDefinitionByName(classNameList[i]);
			}
			// trace(_classList);
		}
		
		public function onAddedToStage(evt:Event):void {
			stage.addEventListener(KeyboardEvent.KEY_DOWN, stageKeyDown);
			stage.addEventListener(KeyboardEvent.KEY_UP, stageKeyUp);
		}
		
		public function drawCanvas(numItems:uint):void {
			var i:uint;
			var item:DisplayObjectContainer;
			var classIndex:uint;
			
			for (i=0; i<numItems; i++) {
				classIndex = Math.floor(Math.random() * _classList.length);
				// trace(classIndex);
				item = new _classList[classIndex]();
				// trace("width = ",item.width);
				randomizeItem(item);
				_itemList.push(item);
				addChild(item);
			}
		}

		public function randomizeCanvas():void {
			// trace("RANDOMIZE canvas");
			var i:uint;
			var max:uint = _itemList.length;
			for(i=0; i<max; i++) {
				randomizeItem(_itemList[i]);
			}
		} 
		public function getItemList():Array {
			return _itemList.slice(0,_itemList.length);
		}
		
		public function randomizeItem(s:DisplayObjectContainer):void {
			var alphaRange:Number = maxAlpha - minAlpha;
			var scaleRange:Number = maxScale - minScale;
			
			// set properties
			s.x = Math.random() * canvasBounds.width + canvasBounds.x;
			s.y = Math.random() * canvasBounds.height + canvasBounds.y;
			
			s.alpha = Math.random() * alphaRange + minAlpha;
			s.scaleX = s.scaleY = Math.random() * scaleRange + minScale;
			s.rotation = Math.random() * randomRotation + baseRotation;
			
			// set mouse interaction
			s.mouseChildren = false;
			s.addEventListener(MouseEvent.MOUSE_DOWN, itemMouseDown);
			s.addEventListener(MouseEvent.MOUSE_UP, itemMouseUp);
		}
		
		public function itemMouseDown(evt:MouseEvent):void {
			if (killItem) {
				var self:DisplayObject = evt.target as DisplayObject;
				var container:DisplayObjectContainer = self.parent as DisplayObjectContainer;
				container.removeChild(self);
			}
			evt.target.startDrag();
		}
		public function itemMouseUp(evt:MouseEvent):void {
			evt.target.stopDrag();
		}
		private function stageKeyDown(evt:KeyboardEvent):void {
			// trace("down");
			if (evt.keyCode == Keyboard.SHIFT ) {
				killItem = true;
			}
		}
		private function stageKeyUp(evt:KeyboardEvent):void {
			if (evt.keyCode == Keyboard.SHIFT ) {
				killItem = false;
			}
		}
	}
}